package dk.nsp.xdscleanup.config;

import org.apache.cxf.binding.soap.SoapHeader;
import org.apache.cxf.binding.soap.SoapMessage;
import org.apache.cxf.binding.soap.interceptor.AbstractSoapInterceptor;
import org.apache.cxf.configuration.jsse.TLSClientParameters;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.headers.Header;
import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import org.apache.cxf.phase.Phase;
import org.apache.cxf.transport.http.HTTPConduit;
import org.openehealth.ipf.commons.ihe.ws.WsTransactionConfiguration;
import org.openehealth.ipf.commons.ihe.xds.core.XdsClientFactory;

import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.xml.namespace.QName;
import java.security.cert.X509Certificate;
import java.util.List;

public abstract class AbstractIheTestConfiguration {
    public static final String XDS_B_2007_TARGET_NAMESPACE = "urn:ihe:iti:xds-b:2007";
    public static final String XDS_B_2010_TARGET_NAMESPACE = "urn:ihe:iti:xds-b:2010";

    private final TrustManager[] trustAllCerts = new TrustManager[] {
            new X509TrustManager() {
                public java.security.cert.X509Certificate[] getAcceptedIssuers() {
                    return null;
                }

                public void checkClientTrusted(X509Certificate[] certs, String authType) {  }

                public void checkServerTrusted(X509Certificate[] certs, String authType) {  }
            }
    };

    protected XdsClientFactory generateXdsRegistryClientFactory(String wsdl, String url, String targetNameSpace, Class<?> clazz){
        final WsTransactionConfiguration WS_CONFIG = new WsTransactionConfiguration(
                new QName(targetNameSpace, "DocumentRegistry_Service",
                        "ihe"), clazz, new QName(
                targetNameSpace,
                "DocumentRegistry_Binding_Soap12", "ihe"), false,
                wsdl, true, false, false, false);

        return new XdsClientFactory(WS_CONFIG, url, null, null,null);
    }

    protected XdsClientFactory generateXdsRepositoryClientFactory(String wsdl, String url, Class<?> clazz){
        final WsTransactionConfiguration WS_CONFIG = new WsTransactionConfiguration(
                new QName(XDS_B_2007_TARGET_NAMESPACE, "DocumentRepository_Service",
                        "ihe"), clazz, new QName(
                XDS_B_2007_TARGET_NAMESPACE,
                "DocumentRepository_Binding_Soap12", "ihe"), true,
                wsdl, true, false, false, false);


        return new XdsClientFactory(WS_CONFIG, url, null, null,null);
    }

    protected void initProxy(Object o) {
        Client proxy = ClientProxy.getClient(o);
        proxy.getOutInterceptors().add(new LoggingOutInterceptor());
        proxy.getInInterceptors().add(new LoggingInInterceptor());
        proxy.getOutInterceptors().add(new DisableMustUnderstandInterceptor());
        HTTPConduit conduit = (HTTPConduit)proxy.getConduit();
        TLSClientParameters tcp = new TLSClientParameters();
        tcp.setTrustManagers(trustAllCerts);
        conduit.setTlsClientParameters(tcp);
    }

    private static class DisableMustUnderstandInterceptor extends AbstractSoapInterceptor {

        public DisableMustUnderstandInterceptor() {
            super(Phase.WRITE);
        }

        @Override
        public void handleMessage(SoapMessage message) throws Fault {
            List<Header> headers = message.getHeaders();
            for (Header header : headers) {
                if (header instanceof SoapHeader) {
                    SoapHeader soapHeader = (SoapHeader) header;
                    soapHeader.setMustUnderstand(false);
                }
            }
        }

    }
}
