
package dk.nsp.xdscleanup.stepdefinitions;

import dk.nsp.xdscleanup.util.documentutil.*;
import io.cucumber.java.en.Given;
import io.cucumber.java.en.Then;
import org.apache.commons.io.FileUtils;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.EbXMLQueryResponse;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.*;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.*;
import org.openehealth.ipf.commons.ihe.xds.core.requests.QueryRegistry;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.FindDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.QueryReturnType;
import org.openehealth.ipf.commons.ihe.xds.core.responses.QueryResponse;
import org.openehealth.ipf.commons.ihe.xds.core.responses.Status;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.lcm.SubmitObjectsRequest;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.query.AdhocQueryRequest;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.query.AdhocQueryResponse;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.rs.RegistryResponseType;
import org.openehealth.ipf.commons.ihe.xds.core.transform.requests.QueryRegistryTransformer;
import org.openehealth.ipf.commons.ihe.xds.core.transform.responses.QueryResponseTransformer;
import org.openehealth.ipf.commons.ihe.xds.iti18.Iti18PortType;
import org.openehealth.ipf.commons.ihe.xds.iti41.Iti41PortType;
import org.openehealth.ipf.commons.ihe.xds.iti43.Iti43PortType;
import org.openehealth.ipf.commons.ihe.xds.iti57.Iti57PortType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static org.junit.Assert.*;

public class
RegisterAndSearchStepDefinitions {
    @Value("${nxrg.document.months.to.keep}")
    private int monthsToKeep;
    private static final String CPR_OID = "1.2.208.176.1.2";
    private final ProvideAndRegisterDocumentSetUtilities provideAndRegisterDocumentSetUtilities = new ProvideAndRegisterDocumentSetUtilities();
    private final Iti41RequestUtilities iti41RequestUtilities = new Iti41RequestUtilities();

    private final Iti57RequestUtilities iti57RequestUtilities = new Iti57RequestUtilities();
    private final EbXMLFactory30 ebXMLFactory = new EbXMLFactory30();
    private final RetrieveDocumentSetUtilities retrieveDocumentSetUtilities = new RetrieveDocumentSetUtilities();

    @Autowired
    private Iti18PortType searchClient;

    @Autowired
    private Iti43PortType retrieveClient;

    @Autowired
    private Iti41PortType registerClient;

    @Autowired
    private Iti57PortType deprecateClient;

    @Autowired
    private String openxdsRepositoryUniqueId;

    private final Map<String, DocumentEntry> documentIdsActive = new LinkedHashMap<>();
    private final Map<String, DocumentEntry> documentIdsExpired = new LinkedHashMap<>();

    @Given("{string} dokument med cpr {string} registreres som en rettelse til forrige")
    public void aktivtDokumentMedCprRegistreresSomRettelse(String documentType, String cpr) throws IOException, URISyntaxException {
        Date serviceStartTime = null; // vil blive defaulted til now
        DocumentEntry lastDocumentEntry = getLastAdded(documentIdsActive);
        DocumentEntry documentEntry = dokumentMedCprRegistreres(documentType, cpr, serviceStartTime, lastDocumentEntry.getEntryUuid());
        documentIdsActive.remove(lastDocumentEntry.getUniqueId());
        documentIdsExpired.put(lastDocumentEntry.getUniqueId(), lastDocumentEntry);
        documentIdsActive.put(documentEntry.getUniqueId(), documentEntry);
    }
    @Given("{string} dokument med cpr {string} registreres")
    public void aktivtDokumentMedCprRegistreres(String documentType, String cpr) throws IOException, URISyntaxException {
        Date serviceStartTime = null; // vil blive defaulted til now
        DocumentEntry documentEntry = dokumentMedCprRegistreres(documentType, cpr, serviceStartTime, null);
        documentIdsActive.put(documentEntry.getUniqueId(), documentEntry);
    }

    @Given("{string} dokument med cpr {string} registreres {int} gange")
    public void aktivtDokumentMedCprRegistreresXAntalgange(String documentType, String cpr, int noOfDocs) throws IOException, URISyntaxException {
        for (int count = 0; count < noOfDocs; count++ ) {
            aktivtDokumentMedCprRegistreres(documentType, cpr);
        }
    }

    @Given("Udløbet {string} dokument med cpr {string} registreres")
    public void udloebetDokumentMedCprRegistreres(String documentType, String cpr) throws IOException, URISyntaxException {
        Date serviceStartTime = getExpiredDate();
        DocumentEntry documentEntry = dokumentMedCprRegistreres(documentType, cpr, serviceStartTime, null);
        documentIdsExpired.put(documentEntry.getUniqueId(), documentEntry);
    }

    @Given("Udløbet {string} dokument med cpr {string} registreres {int} gange")
    public void udloebetDokumentMedCprRegistreresXAntalgange(String documentType, String cpr, int noOfDocs) throws IOException, URISyntaxException {
        for (int count = 0; count < noOfDocs; count++ ) {
            udloebetDokumentMedCprRegistreres(documentType, cpr);
        }
    }

    @Given("Slettemarkeret {string} dokument med cpr {string} registreres")
    public void slettemarkeretDokumentMedCprRegistreres(String documentType, String cpr) throws IOException, URISyntaxException {
        Date serviceStartTime = null; // vil blive defaulted til now
        DocumentEntry documentEntry = dokumentMedCprRegistreres(documentType, cpr, serviceStartTime, null);
        deprecateDocument(documentEntry);
        documentIdsExpired.put(documentEntry.getUniqueId(), documentEntry);
    }
    @Given("Slettemarkeret {string} dokument med cpr {string} registreres {int} gange")
    public void slettemarkretDokumentMedCprRegistreresXAntalgange(String documentType, String cpr, int noOfDocs) throws IOException, URISyntaxException {
        for (int count = 0; count < noOfDocs; count++ ) {
            slettemarkeretDokumentMedCprRegistreres(documentType, cpr);
        }
    }

    @Given("Seneste dokument slettemarkeres")
    public void slettemarkerSenesteDokument() {
        DocumentEntry lastDocumentEntry = getLastAdded(documentIdsActive);
        deprecateDocument(lastDocumentEntry);
        documentIdsActive.remove(lastDocumentEntry.getUniqueId());
        documentIdsExpired.put(lastDocumentEntry.getUniqueId(), lastDocumentEntry);
    }

    private DocumentEntry dokumentMedCprRegistreres(String documentType, String cpr, Date serviceStartAndStopTime, String documentEntryUuidToReplace) throws IOException, URISyntaxException {
        String documentId = provideAndRegisterDocumentSetUtilities.generateId();
        Date now = Calendar.getInstance().getTime();
        if (serviceStartAndStopTime == null) {
            serviceStartAndStopTime = now;
        }

        RequestResponseInformation requestResponseInformation = null;
        switch (documentType) {
            case "Aftale":
                requestResponseInformation = createAftaleDocumentFor(cpr, documentId, now, serviceStartAndStopTime, documentEntryUuidToReplace);
                break;
            case "Labsvar":
                requestResponseInformation = createLabsvarDocumentFor(cpr, documentId, now, serviceStartAndStopTime, documentEntryUuidToReplace);
                break;
        }
        assertNotNull(requestResponseInformation);
        assertEquals(Status.SUCCESS, requestResponseInformation.getResponse().getStatus());
        return requestResponseInformation.getDocumentEntry();
    }

    public void deprecateDocument(DocumentEntry documentEntry) {
        SubmitObjectsRequest request = iti57RequestUtilities.createDeprecateRequest(documentEntry);
        RegistryResponseType response = deprecateClient.documentRegistryUpdateDocumentSet(request);
        assertNotNull(response);
        assertEquals(Status.SUCCESS.getOpcode30(), response.getStatus());
    }

    @Then("Søgning efter dokument med cpr {string} finder registreret dokument")
    public void soegningEfterDokumentMedCprFinderRegistreretDokument(String cpr) {
        List<DocumentEntry> documents = searchDocumentsByCpr(cpr, AvailabilityStatus.APPROVED);
        List<String> documentIdsSearched = getUniqueIdList(documents);
        assertTrue(documentIdsSearched.containsAll(getUniqueIdList(cpr, documentIdsActive)));
        assertTrue(documentIdsSearched.containsAll(getUniqueIdList(cpr, documentIdsExpired)));
    }
    @Then("Søgning efter dokument med cpr {string} finder slettemarkeret dokument")
    public void soegningEfterDokumentMedCprFinderSlettemarkeretDokument(String cpr) {
        List<DocumentEntry> documents = searchDocumentsByCpr(cpr, AvailabilityStatus.DEPRECATED);
        List<String> documentIdsSearched = getUniqueIdList(documents);
        assertTrue(documentIdsSearched.containsAll(getUniqueIdList(cpr, documentIdsExpired)));
    }

    @Then("Søgning efter dokument med cpr {string} finder {int} slettemarkeret dokumenter")
    public void soegningEfterDokumentMedCprFinderSlettemarkeretDokument(String cpr, int noOfDocs) {
        List<DocumentEntry> documents = searchDocumentsByCpr(cpr, AvailabilityStatus.DEPRECATED);
        List<String> documentIdsSearched = getUniqueIdList(documents);
        assertTrue(checkMatching(cpr, documentIdsExpired, documentIdsSearched, noOfDocs));
    }

    @Then("Søgning efter dokument med cpr {string} finder ikke registreret dokument")
    public void soegningEfterDokumentMedCprFinderIkkeRegistreretDokument(String cpr) {
        List<DocumentEntry> documents = searchDocumentsByCpr(cpr, AvailabilityStatus.APPROVED);
        List<String> documentIdsSearched = getUniqueIdList(documents);
        assertFalse(documentIdsSearched.containsAll(getUniqueIdList(cpr, documentIdsActive)));
    }
    @Then("Søgning efter dokument med cpr {string} finder ikke udløbet dokument")
    public void soegningEfterDokumentMedCprFinderIkkeUdloebetDokument(String cpr) {
        List<DocumentEntry> documents = searchDocumentsByCpr(cpr, AvailabilityStatus.APPROVED);
        List<String> documentIdsSearched = getUniqueIdList(documents);
        assertFalse(documentIdsSearched.containsAll(getUniqueIdList(cpr, documentIdsExpired)));
    }
    @Then("Søgning efter dokument med cpr {string} finder ikke slettemarkeret dokument")
    public void soegningEfterDokumentMedCprFinderIkkeSlettemarkeretDokument(String cpr) {
        List<DocumentEntry> documents = searchDocumentsByCpr(cpr, AvailabilityStatus.DEPRECATED);
        List<String> documentIdsSearched = getUniqueIdList(documents);
        assertFalse(documentIdsSearched.containsAll(getUniqueIdList(cpr, documentIdsExpired)));
    }
    @Then("Søgning efter dokument med cpr {string} finder kun ikke-udløbet dokument")
    public void soegningEfterDokumentMedCprFinderRelevantRegistreretDokument(String cpr) {
        List<DocumentEntry> documents = searchDocumentsByCpr(cpr, AvailabilityStatus.APPROVED);
        List<String> documentIdsSearched = getUniqueIdList(documents);
        assertTrue(documentIdsSearched.containsAll(getUniqueIdList(cpr, documentIdsActive)));
        assertFalse(documentIdsSearched.containsAll(getUniqueIdList(cpr, documentIdsExpired)));
    }

    @Then("Hentning af registreret dokument returnerer {int} dokumenter")
    public void hentningAfRegistreretDokumentReturnererXDokumenter(int expectedNumberOfDocuments) {
        List<String> combinedList = Stream.concat(getUniqueIdList(documentIdsActive).stream(), getUniqueIdList(documentIdsExpired).stream()).collect(Collectors.toList());
        RetrieveDocumentSetResponseType response = retrieveDocument(combinedList);
        assertNotNull(response);
        assertEquals(expectedNumberOfDocuments, response.getDocumentResponse().size());
    }

    private RetrieveDocumentSetResponseType retrieveDocument(List<String> documentIds) {
        RetrieveDocumentSetRequestType request = retrieveDocumentSetUtilities.createRetrieveDocumentSetRequestType(documentIds, "urn:oid:1.2.208.176.43210.8.10", openxdsRepositoryUniqueId);
        return retrieveClient.documentRepositoryRetrieveDocumentSet(request);
    }

    private List<DocumentEntry> searchDocumentsByCpr(String cpr, AvailabilityStatus availabilityStatus) {
        AdhocQueryRequest request = createSearchRequest(cpr, availabilityStatus);
        AdhocQueryResponse response = searchClient.documentRegistryRegistryStoredQuery(request);
        return convertResponse(response).getDocumentEntries();
    }

    private AdhocQueryRequest createSearchRequest(String cpr, AvailabilityStatus availabilityStatus) {
        FindDocumentsQuery fdq = new FindDocumentsQuery();
        fdq.setPatientId(new Identifiable(cpr, new AssigningAuthority(CPR_OID)));
        fdq.setStatus(Arrays.asList(availabilityStatus));

        QueryRegistry queryRegistry = new QueryRegistry(fdq);
        queryRegistry.setReturnType(QueryReturnType.LEAF_CLASS);

        return createAdhocQueryRequest(queryRegistry);
    }

    private AdhocQueryRequest createAdhocQueryRequest(QueryRegistry request) {
        QueryRegistryTransformer requestTransformer = new QueryRegistryTransformer();
        EbXMLAdhocQueryRequest30 ebxmlRequest = (EbXMLAdhocQueryRequest30) requestTransformer.toEbXML(request);
        return ebxmlRequest.getInternal();
    }

    private QueryResponse convertResponse(AdhocQueryResponse response) {
        QueryResponseTransformer responseTransformer = new QueryResponseTransformer(ebXMLFactory);
        EbXMLQueryResponse ebXML = new EbXMLQueryResponse30(response);
        return responseTransformer.fromEbXML(ebXML);
    }

    private RequestResponseInformation createAftaleDocumentFor(String patientCpr, String documentId, Date creationDate, Date fromAndTo, String documentEntryUuidToReplace) throws IOException, URISyntaxException {
        DocumentMetadata documentMetadata = iti41RequestUtilities.createDocumentMetadataForAftale(patientCpr, creationDate, fromAndTo);

        // Indlæs dokumentet, som skal registreres
        File documentFile = new File(this.getClass().getResource("/cda/aftale_document.xml").toURI());
        String documentPayload = FileUtils.readFileToString(documentFile);
        documentPayload = documentPayload.replaceAll("¤DOCUMENTID¤", documentId);
        documentPayload = documentPayload.replaceAll("¤PATIENTID¤", patientCpr);

        return createDocumentFor(documentMetadata, documentId, documentPayload, documentEntryUuidToReplace);

    }
    private RequestResponseInformation createLabsvarDocumentFor(String patientCpr, String documentId, Date creationDate, Date fromAndTo, String documentEntryUuidToReplace) throws IOException, URISyntaxException {
        DocumentMetadata documentMetadata = iti41RequestUtilities.createDocumentMetadataForLabsvar(patientCpr, creationDate, fromAndTo);

        // Indlæs dokumentet, som skal registreres
        File documentFile = new File(this.getClass().getResource("/cda/labsvar_dokument_0105500010.xml").toURI());
        String documentPayload = FileUtils.readFileToString(documentFile);
        documentPayload = documentPayload.replaceAll("¤DOCUMENTID¤", documentId);
        documentPayload = documentPayload.replaceAll("¤PATIENTID¤", patientCpr);

        return createDocumentFor(documentMetadata, documentId, documentPayload, documentEntryUuidToReplace);

    }

    private RequestResponseInformation createDocumentFor(DocumentMetadata documentMetadata, String documentId, String documentPayload, String documentEntryUuidToReplace) {

        // Opret documentEntry
        DocumentEntry documentEntry = iti41RequestUtilities.createDocumentEntry(documentId, documentMetadata, null);

        // Opret kald/request
        ProvideAndRegisterDocumentSetRequestType provideAndRegisterDocumentSetRequest = iti41RequestUtilities.buildProvideAndRegisterDocumentSetRequest(documentEntry, documentMetadata, documentPayload, documentEntryUuidToReplace);

        // Udfør kald
        RegistryResponseType registryResponse = registerClient.documentRepositoryProvideAndRegisterDocumentSetB(provideAndRegisterDocumentSetRequest);
        return new RequestResponseInformation(documentEntry, provideAndRegisterDocumentSetUtilities.convertReponse(registryResponse));
    }

    private Date getExpiredDate() {
        Calendar serviceStartTime = Calendar.getInstance();
        serviceStartTime.add(Calendar.MONTH, -(monthsToKeep + 1));
        return serviceStartTime.getTime();
    }

    private List<String>  getUniqueIdList(List<DocumentEntry> documents) {
        //skaffer listen af dokumentid'er fra en lise af documentEntries.
        List<String> documentIds = documents.stream().map(XDSMetaClass::getUniqueId).collect(Collectors.toList());
        return documentIds;
    }
    private List<String> getUniqueIdList(String cpr, Map<String, DocumentEntry> documentMap) {
        //skaffer listen af dokumentid'er, der har det givne cpr nummer
        List<String> documentIdsList = documentMap.entrySet().stream()
                .filter(entry -> cpr.equals(entry.getValue().getPatientId().getId()))
                .map(Map.Entry::getKey)
                .collect(Collectors.toList());
        return documentIdsList;
    }
    private List<String> getUniqueIdList(Map<String, DocumentEntry> documentMap) {
        //skaffer listen af dokumentid'er, uanset cpr nummer
        List<String> documentIdsList = new ArrayList<>(documentMap.keySet());
        return documentIdsList;
    }

    private DocumentEntry getLastAdded(Map<String, DocumentEntry> documentMap) {
        Map.Entry<String, DocumentEntry> lastEntry = null;
        for (Map.Entry<String, DocumentEntry> entry : documentMap.entrySet()) {
            lastEntry = entry;
        }
        return lastEntry.getValue(); //hvis lastEntry er null, så er testen lavet forkert
    }

    private boolean checkMatching(String cpr, Map<String, DocumentEntry> documentMap, List<String> documentList, int noOfMatching) {
        // undersøger hvor mange elementer fra documentMap der findes i documentList. Og hvis det er lig med noOfMatching returneres true eller false.
        List<String> listToCheck = getUniqueIdList(cpr, documentMap);
        int foundCount = 0;
        for (String uniqueId : listToCheck) {
            if (documentList.contains(uniqueId)) {
                foundCount++;
            }
        }
        return noOfMatching == foundCount;
    }

}
