package dk.nsp.xdscleanup.util.documentutil;

import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.ProvideAndRegisterDocumentSetRequestType;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.*;

import java.util.Date;

public class Iti41RequestUtilities extends AbstractXdsUtilities {

	ProvideAndRegisterDocumentSetUtilities provideAndRegisterDocumentSetUtilities = new ProvideAndRegisterDocumentSetUtilities();
	
	public DocumentMetadata createDocumentMetadataForAftale(String patientCpr, Date creationDate, Date serviceStartAndStopTime) {

		DocumentMetadata documentMetadata = createGenericDocumentMetadata(patientCpr, creationDate, serviceStartAndStopTime);
		documentMetadata.setTitle("Dato og tidspunkt for møde mellem patient og sundhedsperson");
		documentMetadata.setFormatCode(Codes.FORMAT_CODE_AFTALEDOKUMENT);
		documentMetadata.setContentTypeCode(Codes.TYPE_CODE_AFTALEDOKUMENT);
		documentMetadata.setTypeCode(Codes.TYPE_CODE_AFTALEDOKUMENT);
		return documentMetadata;
	}
	public DocumentMetadata createDocumentMetadataForLabsvar(String patientCpr, Date creationDate, Date serviceStartAndStopTime) {

		DocumentMetadata documentMetadata = createGenericDocumentMetadata(patientCpr, creationDate, serviceStartAndStopTime);
		documentMetadata.setTitle(Codes.LABSVAR_DISPLAY);
		documentMetadata.setFormatCode(Codes.FORMAT_CODE_LABSVAR);
		documentMetadata.setContentTypeCode(Codes.TYPE_CODE_LABSVAR);
		documentMetadata.setTypeCode(Codes.TYPE_CODE_LABSVAR);
		return documentMetadata;
	}
	private DocumentMetadata createGenericDocumentMetadata(String patientCpr, Date creationDate, Date serviceStartAndStopTime) {

		// Metadata skal overholde Medcoms meta-data profil, som i skrivende stund (feb 2022) findes her: https://svn.medcom.dk/svn/drafts/Standarder/IHE/DK_profil_metadata/Metadata-v096.pdf
		// Attributter, som sættes i metadata skal ligeledes være givet af Medcom i deres lister over værdier: http://svn.medcom.dk/svn/drafts/Standarder/IHE/OID/DK-IHE_Metadata-Common_Code_systems-Value_sets.xlsx

		// Eksempel på udfyldelse af metadata (dokument type specifik udfyldelse ligger i kaldende metoe
		DocumentMetadata documentMetadata = new DocumentMetadata();
		documentMetadata.setPatientId(new Code(patientCpr, new LocalizedString(Codes.CPR_AUTHORITYNAME), Codes.CPR_OID));

		PatientInfo sourcePatientInfo = new PatientInfo();
		sourcePatientInfo.setName(new XpnName("Berggren", "Nancy", "Ann", null, null, null));
		sourcePatientInfo.setGender("F");
		sourcePatientInfo.setDateOfBirth("19481225");
		documentMetadata.setSourcePatientInfo(sourcePatientInfo);

		documentMetadata.setReportTime(creationDate);
		documentMetadata.setOrganisation(new Code("12345679999", new LocalizedString("XDS Testafdeling"), Codes.SOR_OID));
		documentMetadata.setClassCode(new Code("001", new LocalizedString("Klinisk rapport"), "1.2.208.184.100.9"));
		documentMetadata.setHealthcareFacilityTypeCode(new Code("22232009", new LocalizedString("hospital"), "2.16.840.1.113883.6.96"));
		documentMetadata.setPracticeSettingCode(new Code("408443003", new LocalizedString("almen medicin"), "2.16.840.1.113883.6.96"));
		documentMetadata.setSubmissionTime(creationDate);
		documentMetadata.setServiceStartTime(serviceStartAndStopTime);
		documentMetadata.setServiceStopTime(serviceStartAndStopTime);
		documentMetadata.setMimeType("text/xml");
		documentMetadata.setLanguageCode("da-DK");
		documentMetadata.setConfidentialityCode(new Code("N", new LocalizedString("N"), "2.16.840.1.113883.5.25"));
		return documentMetadata;
	}

	public DocumentEntry createDocumentEntry(String documentId, DocumentMetadata documentMetadata, String documentEntryUuid) {
		documentEntryUuid = documentEntryUuid == null ? generateUUIDwithPrefix() : documentEntryUuid;
		return provideAndRegisterDocumentSetUtilities.createDocumentEntry(documentMetadata, documentEntryUuid, documentId, documentId);
	}
	
	public ProvideAndRegisterDocumentSetRequestType buildProvideAndRegisterDocumentSetRequest(DocumentEntry documentEntry, DocumentMetadata documentMetadata, String documentPayload, String documentEntryUuidToReplace) {
		
		return provideAndRegisterDocumentSetUtilities.buildProvideAndRegisterDocumentSetRequest(documentEntry, documentPayload, documentMetadata, documentEntryUuidToReplace);
	}

}
