package dk.nsp.xdscleanup.util.documentutil;

import org.joda.time.DateTime;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.*;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RegisterDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.lcm.SubmitObjectsRequest;

public class Iti57RequestUtilities extends AbstractXdsUtilities {

	public SubmitObjectsRequest createDeprecateRequest(DocumentEntry toDeprecate) {
		return createDeprecateDocumentRequest(toDeprecate, toDeprecate.getEntryUuid());
	}

	private SubmitObjectsRequest createDeprecateDocumentRequest(DocumentEntry documentEntryToDeprecate, String uniqueId) {

		String submissionSetid = generateUUIDwithPrefix();

		SubmissionSet ss = createSubmissionSetForDocumentEntry(submissionSetid, documentEntryToDeprecate, uniqueId);
		Association as = createAssociateDeprecateStatus(ss, documentEntryToDeprecate);

		RegisterDocumentSet request = new RegisterDocumentSet();
		request.setSubmissionSet(ss);
		request.getAssociations().add(as);
		
		return createSubmitObjectsRequest(request);
	}

	private SubmissionSet createSubmissionSetForDocumentEntry(String id, DocumentEntry documentEntry, String uniqueId) {
		SubmissionSet submissionSet = new SubmissionSet();
		submissionSet.setEntryUuid(id);
		submissionSet.setLogicalUuid(id);
		submissionSet.setUniqueId(documentEntry.getEntryUuid());
		submissionSet.setUniqueId(uniqueId);
		submissionSet.setPatientId(documentEntry.getPatientId());
		submissionSet.setSourceId(documentEntry.getRepositoryUniqueId());
		submissionSet.setContentTypeCode(documentEntry.getTypeCode());
		submissionSet.setSubmissionTime(new DateTime());

		return submissionSet;
	}

	public Association createAssociateDeprecateStatus(SubmissionSet submissionSet, DocumentEntry toDeprecate) {
		Association a = new Association();

		a.setEntryUuid(generateUUID());
		a.setAssociationType(AssociationType.UPDATE_AVAILABILITY_STATUS);
		a.setSourceUuid(submissionSet.getLogicalUuid());
		a.setTargetUuid(toDeprecate.getEntryUuid());
		a.setOriginalStatus(toDeprecate.getAvailabilityStatus());
		a.setNewStatus(AvailabilityStatus.DEPRECATED);
		
		return a;
	}
}
